import datetime
from pathlib import Path

from zimscraperlib.zim import Creator

from fcc2zim.build import build_command
from fcc2zim.constants import FCC_LANG_MAP, VERSION, Global
from fcc2zim.fetch import fetch_command
from fcc2zim.prebuild import prebuild_command
from fcc2zim.zimscraperlib_fork import compute_descriptions


class Scraper:
    def __init__(
        self,
        *,
        do_fetch: bool,
        do_prebuild: bool,
        do_build: bool,
        zimui_dist: str,
        output: str,
        build: str,
        language: str,
        name: str,
        title: str,
        description: str,
        long_description: str | None,
        content_creator: str,
        publisher: str,
        zim_file: str | None,
        force: bool,
        course_csv: str,
        zip_path: str | None,
        start_date: datetime.date,
    ):
        self.creator = None

        self.do_fetch = do_fetch
        self.do_prebuild = do_prebuild
        self.do_build = do_build

        if not (self.do_fetch + self.do_prebuild + self.do_build):
            self.do_fetch = self.do_prebuild = self.do_build = True

        self.zimui_dist = Path(zimui_dist)
        if not self.zimui_dist.exists():
            raise ValueError(f"zimui_dist directory {self.zimui_dist} does not exists")

        self.output = Path(output)
        self.build = Path(build)
        self.curriculum_raw = self.build.joinpath("curriculum-raw")
        self.curriculum_dist = self.build.joinpath("curriculum-dist")

        # Make sure the output directory exists
        self.output.mkdir(parents=True, exist_ok=True)
        self.build.mkdir(parents=True, exist_ok=True)

        self.language = language
        if self.language not in FCC_LANG_MAP:
            raise ValueError(f"Unsupported language {self.language}")
        self.fcc_lang = FCC_LANG_MAP[language]

        self.name = name
        self.title = title

        self.description = description
        self.long_description = long_description
        self.description, self.long_description = compute_descriptions(
            self.description, self.description, self.long_description
        )

        self.content_creator = content_creator
        self.publisher = publisher
        self.force = force
        self.course_csv = course_csv
        if not zip_path:
            self.zip_path = self.build.joinpath("main.zip")
        else:
            self.zip_path = Path(zip_path)
            if not self.zip_path.exists():
                raise ValueError(f"Zip file not found in {self.zip_path}")

        # if we do not build the ZIM, we can stop here
        if not self.do_build:
            return

        period = start_date.strftime("%Y-%m")
        if zim_file:
            self.zim_path = Path(zim_file.format(period=period))
            # make sure we were given a filename and not a path
            if Path(self.zim_path.name) != self.zim_path:
                raise ValueError(f"zim_name is not a filename: {zim_file}")
        else:
            self.zim_path = Path(f"{name}_{period}.zim")

        # build full path
        self.zim_path = self.output.joinpath(self.zim_path)

        if self.zim_path.exists():
            if not self.force:
                raise ValueError(f"ZIM file {self.zim_path} already exist.")
            Global.logger.info(f"Removing existing ZIM file {self.zim_path}")
            self.zim_path.unlink()
        else:
            Global.logger.info(f"ZIM path: {self.zim_path}")

        logo_path = Path(__file__).parent.joinpath("assets", "fcc_48.png")
        if not logo_path.exists():
            raise ValueError(f"Logo not found at {logo_path}")

        self.creator = Creator(self.zim_path, "index.html").config_metadata(
            Name=self.name,
            Title=self.title,
            Publisher=self.publisher,
            Date=start_date,
            Creator=self.content_creator,
            Description=self.description,
            LongDescription=self.long_description,
            Language=self.language,
            Tags=";".join(["FCC", "freeCodeCamp"]),
            Scraper=f"fcc2zim v{VERSION}",
            Illustration_48x48_at_1=logo_path.read_bytes(),
        )

        # start creator early to detect any problem early as well
        self.creator.start()

    def run(self):
        try:
            self.run_commands()
        except Exception as exc:
            if self.creator:
                self.creator.can_finish = False
            if isinstance(exc, KeyboardInterrupt):
                Global.logger.error("KeyboardInterrupt, exiting.")
                raise SystemExit(3) from exc
            else:
                Global.logger.error(f"Interrupting process due to error: {exc}")
                Global.logger.exception(exc)
                raise SystemExit(2) from exc
        else:
            if self.creator:
                self.creator.finish()
                Global.logger.info(f"Finished creating Zim at {self.zim_path}")

    def run_commands(self):
        if self.do_fetch:
            fetch_command(
                force=self.force,
                curriculum_raw=self.curriculum_raw,
                zip_path=self.zip_path,
            )
        if self.do_prebuild:
            prebuild_command(
                fcc_lang=self.fcc_lang,
                course_csv=self.course_csv,
                curriculum_raw=self.curriculum_raw,
                curriculum_dist=self.curriculum_dist,
            )
        if self.do_build:
            build_command(
                fcc_lang=self.fcc_lang,
                creator=self.creator,
                zimui_dist=self.zimui_dist,
                curriculum_dist=self.curriculum_dist,
            )



import shutil
import zipfile
from pathlib import Path

import requests

from fcc2zim.constants import Global


def fetch_command(zip_path: Path, curriculum_raw: Path, *, force: bool):
    Global.logger.info("Scraper: fetch phase starting")
    url = "https://github.com/freeCodeCamp/freeCodeCamp/archive/refs/heads/main.zip"

    # Don't redownload the file if we already have it (it's a large file)
    if force or not zip_path.exists():
        Global.logger.debug(f"Download zip file to {zip_path}")
        resp = requests.get(url, allow_redirects=True, timeout=5)
        zip_path.write_bytes(resp.content)
    else:
        Global.logger.debug(f"Using existing zip file {zip_path}")

    curriculum_raw.mkdir(parents=True, exist_ok=True)
    shutil.rmtree(curriculum_raw)

    Global.logger.debug("Extracting files")
    with zipfile.ZipFile(zip_path, "r") as zip_ref:
        members = [
            member
            for member in zip_ref.namelist()
            if member.startswith("freeCodeCamp-main/curriculum/")
            or member.startswith("freeCodeCamp-main/client/i18n/locales")
        ]
        zip_ref.extractall(members=members, path=curriculum_raw)
        Global.logger.info(f"Extracted {len(members)} files")
    Global.logger.info(f"Fetched curriculum into {curriculum_raw}")
    Global.logger.info("Scraper: fetch phase finished")



import pathlib

import yaml


# Each markdown challenge contains 'frontmatter' which is a header
# consisting of yaml formatted data enclosed inside of '---\n' lines
# '---' lines in YAML denote multiple documents, so this can be read
# directly with python's yaml library pulling in the first document
# with 'next' and immediately returning
def read_yaml_frontmatter(filename: pathlib.Path):
    with open(filename) as f:
        return next(yaml.load_all(f, Loader=yaml.FullLoader))


class Challenge:
    def __init__(self, fpath: str | pathlib.Path) -> None:
        self.path = pathlib.Path(fpath)
        self.course_slug = self.path.parent.stem
        self.course_superblock = "-".join(self.path.parent.parent.stem.split("-")[1:])
        self.language = self.path.parent.parent.parent.stem
        self._frontmatter = None

    def identifier(self):
        return str(self.frontmatter()["id"])

    def title(self):
        return str(self.frontmatter()["title"])

    def frontmatter(self):
        if not self._frontmatter:
            self._frontmatter = read_yaml_frontmatter(self.path)
        return self._frontmatter



import argparse
import datetime
import functools
import os

from zimscraperlib.constants import (
    MAXIMUM_DESCRIPTION_METADATA_LENGTH as MAX_DESC_LENGTH,
)
from zimscraperlib.constants import (
    MAXIMUM_LONG_DESCRIPTION_METADATA_LENGTH as MAX_LONG_DESC_LENGTH,
)

from fcc2zim.constants import FCC_LANG_MAP, VERSION, Global, set_debug
from fcc2zim.scraper import Scraper


def log_and_sys_exit(func):
    @functools.wraps(func)
    def wrapper():
        try:
            func()
        except SystemExit:  # SystemExit has been asked for at lower level, simply do it
            raise
        except Exception as exc:
            Global.logger.error(f"A fatal error occurred: {exc}")
            Global.logger.exception(exc)
            raise SystemExit(1) from exc

    return wrapper


@log_and_sys_exit
def main():
    parser = argparse.ArgumentParser(
        prog="fcc2zim",
        description="Scraper to create ZIM files from freeCodeCamp courses",
    )

    parser.add_argument(
        "--course",
        type=str,
        help="Course or course list (separated by commas)",
        required=True,
    )
    parser.add_argument(
        "--language",
        type=str,
        help="Curriculum language",
        required=True,
        choices=FCC_LANG_MAP.keys(),
    )
    parser.add_argument(
        "--name",
        type=str,
        help="ZIM name. Used as identifier and filename (date will be appended)",
        required=True,
    )
    parser.add_argument(
        "--title",
        type=str,
        # once Zimscraperlib > 3.1.1 is released, use constant from library
        # instead of '30' magic number
        help="Title of zim file (less than 30 chars)",
        required=True,
    )
    parser.add_argument(
        "--description",
        type=str,
        help=f"Description of ZIM file (less than {MAX_DESC_LENGTH} chars)",
        required=True,
    )
    parser.add_argument(
        "--long-description",
        type=str,
        help=f"Long description of ZIM file (less than {MAX_LONG_DESC_LENGTH} chars)",
    )
    parser.add_argument(
        "--creator",
        type=str,
        help="Name of freeCodeCamp courses creator",
        default="freeCodeCamp",
    )
    parser.add_argument(
        "--publisher", type=str, help="Publisher of the zim file", default="OpenZIM"
    )
    parser.add_argument(
        "--force",
        help="Force a full reprocessing, not benefiting from any cached file",
        action="store_true",
        default=False,
    )
    parser.add_argument(
        "--debug",
        help="Enable verbose output",
        action="store_true",
        default=False,
    )
    parser.add_argument(
        "--output",
        type=str,
        help="Output directory where zim file will be built",
        default=os.getenv("FCC_OUTPUT", "../output"),
    )
    parser.add_argument(
        "--build",
        type=str,
        help="The build directory to hold temporary files during scraper operation",
        default=os.getenv("FCC_BUILD", "../build"),
    )
    parser.add_argument(
        "--zimui-dist",
        type=str,
        help=(
            "Directory containing Vite build output from the Zim UI Vue.JS application"
        ),
        default=os.getenv("FCC_ZIMUI_DIST", "../zimui/dist"),
    )
    parser.add_argument(
        "--zim-file",
        type=str,
        help="ZIM file name (based on --name if not provided), could contain {period}"
        " placeholder which will be replaced by <year>_<month>",
    )
    parser.add_argument(
        "--zip-path",
        help="Path to zip file containing FCC courses",
        type=str,
    )
    parser.add_argument(
        "--version",
        help="Display scraper version and exit",
        action="version",
        version=f"fcc2zim {VERSION}",
    )

    args = parser.parse_args()

    Global.logger.info(f"Starting fcc2zim {VERSION}")

    set_debug(debug=args.debug)

    scraper = Scraper(
        do_fetch=os.getenv("DO_FETCH", "False").lower() == "true",
        do_prebuild=os.getenv("DO_PREBUILD", "False").lower() == "true",
        do_build=os.getenv("DO_BUILD", "False").lower() == "true",
        zimui_dist=args.zimui_dist,
        output=args.output,
        build=args.build,
        language=args.language,
        name=args.name,
        title=args.title,
        description=args.description,
        long_description=args.long_description,
        content_creator=args.creator,
        publisher=args.publisher,
        zim_file=args.zim_file,
        force=args.force,
        course_csv=args.course,
        zip_path=args.zip_path,
        start_date=datetime.date.today(),
    )

    scraper.run()

    Global.logger.info("Scraper completed")



import logging

from zimscraperlib.logging import getLogger

from fcc2zim.__about__ import __version__

FCC_LANG_MAP = {
    "ara": "arabic",
    "cmn": "chinese",
    "lzh": "chinese-traditional",
    "eng": "english",
    "spa": "espanol",
    "deu": "german",
    "ita": "italian",
    "jpn": "japanese",
    "por": "portuguese",
    "ukr": "ukranian",
}

VERSION = __version__


class Global:
    debug = False
    logger: logging.Logger = getLogger("fcc2zim", level=logging.INFO)


def set_debug(*, debug: bool):
    Global.debug = debug
    Global.logger = getLogger(  # refresh logger to update log level
        "fcc2zim", level=logging.DEBUG if Global.debug else logging.INFO
    )



import datetime
from pathlib import Path
from tempfile import NamedTemporaryFile, TemporaryDirectory

import pytest

from fcc2zim.scraper import Scraper

DEFAULT_START_DATE = datetime.date.fromisoformat("2023-08-23")
WORKING_DIR = TemporaryDirectory(prefix="fcc2zim_tests_")
WORKING_DIR_PATH = Path(WORKING_DIR.name)
ZIMUI_DIST_PATH = WORKING_DIR_PATH.joinpath("zimui/dist")
ZIMUI_DIST_PATH.mkdir(parents=True, exist_ok=True)
BUILD_PATH = WORKING_DIR_PATH.joinpath("build")
OUTPUT_PATH = WORKING_DIR_PATH.joinpath("output")

LONG_TEXT = (
    "Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor "
    "incididunt ut labore et dolore magna aliqua. At erat pellentesque adipiscing "
    "commodo elit at imperdiet. Rutrum tellus pellentesque eu tincidunt tortor aliquam"
    " nulla facilisi. Eget lorem dolor sed viverra ipsum nunc. Ipsum nunc aliquet "
    "bibendum enim facilisis gravida neque convallis. Aliquam malesuada bibendum arcu "
    "vitae elementum curabitur. Platea dictumst quisque sagittis purus sit amet "
    "volutpat. Blandit libero volutpat sed cras ornare. In eu mi bibendum neque "
    "egestas. Egestas dui id ornare arcu odio. Pulvinar neque laoreet suspendisse "
    "interdum. Fames ac turpis egestas integer eget aliquet nibh praesent tristique. Et"
    " egestas quis ipsum suspendisse ultrices gravida dictum fusce. Malesuada fames ac "
    "turpis egestas. Tincidunt nunc pulvinar sapien et ligula ullamcorper malesuada "
    "proin libero. In arcu cursus euismod quis viverra. Faucibus in ornare quam viverra"
    ". Curabitur vitae nunc sed velit dignissim sodales ut eu sem. Velit scelerisque in"
    " dictum non consectetur a erat nam. Proin fermentum leo vel orci porta non. Fames"
    " ac turpis egestas sed tempus. Vitae justo eget magna fermentum iaculis eu non. "
    "Imperdiet massa tincidunt nunc pulvinar sapien et ligula. Laoreet sit amet cursus "
    "sit amet dictum sit amet. Quis hendrerit dolor magna eget. Orci ac auctor augue "
    "mauris augue. Consequat interdum varius sit amet mattis. At ultrices mi tempus "
    "imperdiet nulla malesuada pellentesque elit. Volutpat est velit egestas dui. "
    "Potenti nullam ac tortor vitae. At tempor commodo ullamcorper a lacus vestibulum "
    "sed arcu non. Duis ut diam quam nulla. Vestibulum mattis ullamcorper velit sed "
    "ullamcorper. Sit amet commodo nulla facilisi nullam vehicula. Faucibus purus in "
    "massa tempor nec feugiat. Sem fringilla ut morbi tincidunt augue interdum velit. "
    "Etiam dignissim diam quis enim lobortis scelerisque fermentum dui. Nunc vel risus "
    "commodo viverra maecenas accumsan. Aenean sed adipiscing diam donec adipiscing "
    "tristique. Maecenas accumsan lacus vel facilisis volutpat est velit egestas. Nulla"
    " aliquet porttitor lacus luctus accumsan tortor posuere ac. Habitant morbi "
    "tristique senectus et netus et. Eget mi proin sed libero enim sed faucibus turpis "
    "in. Vulputate enim nulla aliquet porttitor lacus. Dui ut ornare lectus sit amet "
    "est. Quam lacus suspendisse faucibus interdum posuere. Sagittis orci a scelerisque"
    " purus semper eget duis at tellus. Tellus molestie nunc non blandit massa. Feugiat"
    " vivamus at augue eget arcu dictum varius duis at. Varius morbi enim nunc faucibus"
    " a pellentesque sit. Id aliquet lectus proin nibh nisl condimentum id venenatis a."
    " Tortor dignissim convallis aenean et tortor at risus viverra adipiscing. Aliquam "
    "malesuada bibendum arcu vitae elementum curabitur vitae nunc sed. Habitasse platea"
    " dictumst quisque sagittis purus sit amet volutpat. Vitae auctor eu augue ut "
    "lectus. At varius vel pharetra vel turpis nunc eget. Dictum at tempor  commodo "
    "ullamcorper a lacus vestibulum sed arcu. Pellentesque massa placerat duis "
    "ultricies. Enim nunc faucibus a pellentesque sit amet porttitor eget dolor. "
    "Volutpat blandit aliquam etiam erat velit scelerisque in. Amet mattis vulputate "
    "enim nulla aliquet porttitor. Egestas maecenas pharetra convallis posuere morbi "
    "leo urna molestie. Duis ut diam quam nulla porttitor massa id. In fermentum "
    "posuere urna nec tincidunt praesent. Turpis egestas sed tempus urna et pharetra "
    "pharetra massa. Tellus molestie nunc non blandit massa. Diam phasellus vestibulum "
    "lorem sed risus ultricies. Egestas erat imperdiet sed euismod nisi porta lorem. "
    "Quam viverra orci sagittis eu volutpat odio facilisis mauris sit. Ornare aenean "
    "euismod elementum nisi quis. Laoreet non curabitur gravida arcu ac tortor "
    "dignissim convallis aenean. Sagittis aliquam malesuada bibendum arcu vitae "
    "elementum. Sed blandit libero volutpat sed cras ornare. Sagittis eu volutpat odio "
    "facilisis mauris. Facilisis volutpat est velit egestas dui id ornare arcu odio. "
    "Eu feugiat pretium  nibh."
)


class TestScraper:
    def create_scraper(
        self,
        *,
        do_fetch: bool = True,
        do_prebuild: bool = True,
        do_build: bool = True,
        zimui_dist: str = str(ZIMUI_DIST_PATH),
        output: str = str(OUTPUT_PATH),
        build: str = str(BUILD_PATH),
        language: str = "eng",
        name="fcc_en_javascript",
        title="freeCodeCamp Javascript",
        description="FCC Javascript Courses",
        long_description: str | None = None,
        content_creator: str = "freeCodeCamp",
        publisher="openZIM",
        zim_file: str | None = None,
        force: bool = False,
        course_csv="regular-expressions,basic-javascript",
        zip_path: str | None = None,
        start_date: datetime.date = DEFAULT_START_DATE,
    ):
        return Scraper(
            do_fetch=do_fetch,
            do_prebuild=do_prebuild,
            do_build=do_build,
            zimui_dist=zimui_dist,
            output=output,
            build=build,
            language=language,
            name=name,
            title=title,
            description=description,
            long_description=long_description,
            content_creator=content_creator,
            publisher=publisher,
            zim_file=zim_file,
            force=force,
            course_csv=course_csv,
            zip_path=zip_path,
            start_date=start_date,
        )

    def test_init_ok(self):
        assert not OUTPUT_PATH.exists()
        assert not BUILD_PATH.exists()
        self.create_scraper()
        assert OUTPUT_PATH.exists()
        assert BUILD_PATH.exists()

    @pytest.mark.parametrize(
        "do_fetch, do_prebuild, do_build, expected_do_fetch, expected_do_prebuild,"
        "expected_do_build",
        [
            pytest.param(False, False, False, True, True, True, id="FFF"),
            pytest.param(True, False, False, True, False, False, id="TFF"),
            pytest.param(False, True, False, False, True, False, id="FTF"),
            pytest.param(True, True, False, True, True, False, id="TTF"),
            pytest.param(False, False, True, False, False, True, id="FFT"),
            pytest.param(True, False, True, True, False, True, id="TFT"),
            pytest.param(False, True, True, False, True, True, id="FTT"),
            pytest.param(True, True, True, True, True, True, id="TTT"),
        ],
    )
    def test_do_phases_ok(
        self,
        *,
        do_fetch: bool,
        do_prebuild: bool,
        do_build: bool,
        expected_do_fetch: bool,
        expected_do_prebuild: bool,
        expected_do_build: bool,
    ):
        scraper = self.create_scraper(
            do_fetch=do_fetch, do_prebuild=do_prebuild, do_build=do_build
        )
        assert scraper.do_fetch == expected_do_fetch
        assert scraper.do_prebuild == expected_do_prebuild
        assert scraper.do_build == expected_do_build

    def test_zimui_dist_ko(self):
        with pytest.raises(ValueError):
            self.create_scraper(zimui_dist="whatever")

    @pytest.mark.parametrize(
        "language, expected_fcc_lang",
        [
            pytest.param("eng", "english", id="english"),
            pytest.param("eng", "english", id="english"),
            pytest.param("ara", "arabic", id="arabic"),
            pytest.param("cmn", "chinese", id="chinese"),
            pytest.param("lzh", "chinese-traditional", id="chinese-traditional"),
            pytest.param("eng", "english", id="english"),
            pytest.param("spa", "espanol", id="espanol"),
            pytest.param("deu", "german", id="german"),
            pytest.param("ita", "italian", id="italian"),
            pytest.param("jpn", "japanese", id="japanese"),
            pytest.param("por", "portuguese", id="portuguese"),
            pytest.param("ukr", "ukranian", id="ukranian"),
        ],
    )
    def test_fcc_lang_ok(self, language: str, expected_fcc_lang: str):
        scraper = self.create_scraper(language=language)
        assert scraper.language == language
        assert scraper.fcc_lang == expected_fcc_lang

    def test_language_ko(self):
        with pytest.raises(ValueError):
            self.create_scraper(language="whatever")

    def test_description_ko(self):
        with pytest.raises(ValueError):
            self.create_scraper(description=LONG_TEXT[:81])

    def test_long_description_ko(self):
        with pytest.raises(ValueError):
            self.create_scraper(long_description=LONG_TEXT[:4001])

    def test_title_ko(self):
        with pytest.raises(ValueError):
            self.create_scraper(title=LONG_TEXT[:31])

    def test_zip_path_ok(self):
        with NamedTemporaryFile(dir=WORKING_DIR_PATH) as tmp:
            zip_path = tmp.name
            self.create_scraper(zip_path=zip_path)

    def test_zip_path_ko(self):
        with pytest.raises(ValueError):
            self.create_scraper(zip_path="whatever")

    @pytest.mark.parametrize(
        "name, start_date",
        [
            pytest.param("something", "2023-08-23", id="case1"),
            pytest.param("name2", "2023-08-24", id="case2"),
        ],
    )
    def test_zim_file_default(self, name, start_date):
        scraper = self.create_scraper(
            name=name, start_date=datetime.date.fromisoformat(start_date)
        )
        assert scraper.zim_path == OUTPUT_PATH.joinpath(f"{name}_{start_date[:7]}.zim")

    def test_zim_file_is_path_ko(self):
        with pytest.raises(ValueError):
            self.create_scraper(zim_file=str(OUTPUT_PATH.joinpath("whatever.zim")))

    def test_zim_file_ok(self):
        self.create_scraper(zim_file="whatever.zim")

    def test_zim_file_exists_ko(self):
        with NamedTemporaryFile(dir=OUTPUT_PATH, suffix=".zim") as tmp:
            zim_file = Path(tmp.name).name
            with pytest.raises(ValueError):
                self.create_scraper(zim_file=zim_file)

    def test_zim_file_exists_force(self):
        with NamedTemporaryFile(dir=OUTPUT_PATH, suffix=".zim", delete=False) as tmp:
            zim_file = Path(tmp.name).name
            self.create_scraper(zim_file=zim_file, force=True)
            assert not Path(tmp.name).exists()



import json
from collections import OrderedDict
from pathlib import Path

from zimscraperlib.zim import Creator

from fcc2zim.constants import Global


def build_curriculum_redirects(curriculum_dist: Path, fcc_lang: str):
    """
    Build the list of redirects from challenge URL to Vite hash URL

    The Vite app uses its own router to navigate. We have a single HTML file, but we
    need an URL for each challenge for the zim search to work.
    This builds the list of redirect needed fron the challenge URL to Vite hash URL.
    """
    index_json_path = curriculum_dist.joinpath("curriculum", fcc_lang, "index.json")
    with open(index_json_path) as course_index_str:
        superblock_dict = json.load(course_index_str)[fcc_lang]

    redirects = []
    for superblock in superblock_dict:
        course_list = superblock_dict[superblock]
        for course in course_list:
            meta_json_path = Path(
                curriculum_dist,
                "curriculum",
                fcc_lang,
                superblock,
                course,
                "_meta.json",
            )
            challenges = json.loads(meta_json_path.read_text())["challenges"]
            for challenge in challenges:
                title = challenge["title"]
                redirects.append(
                    (f'{fcc_lang}/{superblock}/{course}/{challenge["slug"]}', title)
                )

    return OrderedDict(redirects).items()


def build_command(
    zimui_dist: Path,
    fcc_lang: str,
    creator: Creator,
    curriculum_dist: Path,
):
    Global.logger.info("Scraper: build phase starting")

    # Add zimui files
    for file in zimui_dist.rglob("*"):
        if file.is_dir():
            continue
        path = str(Path(file).relative_to(zimui_dist))
        Global.logger.debug(f"Adding {path} to ZIM")
        creator.add_item_for(path, fpath=file)

    # Add prebuild generated curriculum file
    for file in curriculum_dist.rglob("*"):
        if file.is_dir():
            continue
        path = str(Path("fcc").joinpath(Path(file).relative_to(curriculum_dist)))
        Global.logger.debug(f"Adding {path} to ZIM")
        creator.add_item_for(path, fpath=file)

    for redir_slug, redir_title in build_curriculum_redirects(
        curriculum_dist=curriculum_dist, fcc_lang=fcc_lang
    ):
        redirect_path = f"{redir_slug}"
        redirect_url = redir_slug.count("/") * "../" + f"index.html#{redir_slug}"
        content = (
            f"<html><head><title>{redir_title}</title>"
            f'<meta http-equiv="refresh" content="0;URL=\'{redirect_url}\'" />'
            f"</head><body></body></html>"
        )
        Global.logger.debug(
            f"Redirecting {redirect_path} to {redirect_url} for slug {redir_slug}"
            f"and title {redir_title}",
        )
        creator.add_item_for(
            redirect_path,
            content=bytes(content, "utf-8"),
            title=redir_title,
            mimetype="text/html",
            is_front=True,
        )
        # Example index.html#/english/regular-expressions/extract-matches

    Global.logger.info("Scraper: build phase finished")



import json
import shutil
from pathlib import Path

from fcc2zim.challenge import Challenge
from fcc2zim.constants import Global


def get_challenges_for_lang(tmp_path, language="english"):
    return Path(tmp_path, language).rglob("*.md")


def update_index(path: Path, superblock: str, slug: str, language="english"):
    index_path = path.joinpath("index.json")
    if not index_path.exists():
        index_path.write_bytes(json.dumps({}).encode("utf-8"))

    index = json.loads(index_path.read_text())
    if language not in index:
        index[language] = {}
    if superblock not in index[language]:
        index[language][superblock] = []
    if slug not in index[language][superblock]:
        index[language][superblock].append(slug)

    with open(index_path, "w") as outfile:
        json.dump(index, outfile, indent=4)


"""
Simply copies over the locales to the client locales path
"""


def write_locales_to_path(source: Path, curriculumdir: Path, language="english"):
    shutil.copytree(source, curriculumdir / "locales" / language)


def write_course_to_path(
    challenge_list: list[Challenge],
    superblock: str,
    course_slug: str,
    curriculumdir: Path,
):
    """Writes the course to the chosen path.

    Each individual Markdown challenge is written along
    with the meta data for the course.

    Finally, we udpate the root index.json file with the course, which allows
    us to render a page listing all available courses
    """
    curriculumdir.mkdir(parents=True, exist_ok=True)
    meta = {"challenges": []}

    for challenge in challenge_list:
        challenge_dest_path = curriculumdir.joinpath(
            challenge.course_superblock, challenge.course_slug
        )
        challenge_dest_path.mkdir(parents=True, exist_ok=True)
        shutil.copy2(challenge.path, challenge_dest_path.joinpath(challenge.path.name))
        meta["challenges"].append(
            {"title": challenge.title(), "slug": challenge.path.stem}
        )

    meta_path = curriculumdir.joinpath(superblock, course_slug, "_meta.json")
    meta_path.parent.mkdir(parents=True, exist_ok=True)
    with open(meta_path, "w") as outfile:
        json.dump(meta, outfile, indent=4)

    # Create an index with a list of the courses
    update_index(curriculumdir, superblock, course_slug, challenge_list[0].language)


def prebuild_command(
    course_csv: str,
    fcc_lang: str,
    curriculum_raw: Path,
    curriculum_dist: Path,
):
    """Transform raw data in curriculum_raw directory into pre-built data in
    curriculum_dist directory

    E.g. if lang in english:
    - <curriculum_dist>/index.json
        => { 'english': {'superblock': ['basic-javascript'] } }
    - <curriculum_dist>/english/<superblock>/<course_slug>/_meta.json
        => { challenges: [{slug, title}] }
    - <curriculum_dist>/english/<superblock>/<course_slug>/{slug}.md
    """
    Global.logger.info("Scraper: prebuild phase starting")

    curriculum_dist.mkdir(parents=True, exist_ok=True)
    shutil.rmtree(curriculum_dist)

    challenges = curriculum_raw.joinpath(
        "freeCodeCamp-main", "curriculum", "challenges"
    )
    locales = curriculum_raw.joinpath(
        "freeCodeCamp-main", "client", "i18n", "locales", fcc_lang
    )

    # eg. ['basic-javascript', 'debugging']
    for course in course_csv.split(","):
        Global.logger.debug(f"Prebuilding {course}")
        meta = json.loads(challenges.joinpath("_meta", course, "meta.json").read_text())
        # Get the order that the challenges should be completed in for <course>
        ids = [
            item[0] if isinstance(item, list) else item["id"]
            for item in meta["challengeOrder"]
        ]
        superblock = meta["superBlock"]

        challenge_list: list[Challenge] = []
        for file in get_challenges_for_lang(challenges, fcc_lang):
            challenge = Challenge(file)
            if challenge.course_superblock != superblock:
                continue
            # ID is a UUID the Challenge, the only add it to the challenge list if it's
            # a part of the course.
            if challenge.identifier() in ids:
                challenge_list.append(challenge)

        write_course_to_path(
            sorted(challenge_list, key=lambda x: ids.index(x.identifier())),
            superblock,
            course,
            curriculum_dist.joinpath("curriculum", fcc_lang),
        )

    # Copy all the locales for this language
    write_locales_to_path(locales, curriculum_dist, fcc_lang)
    Global.logger.info(f"Prebuilt curriculum into {curriculum_dist}")
    Global.logger.info("Scraper: prebuild phase finished")



from zimscraperlib.constants import (
    MAXIMUM_DESCRIPTION_METADATA_LENGTH as MAX_DESC_LENGTH,
)
from zimscraperlib.constants import (
    MAXIMUM_LONG_DESCRIPTION_METADATA_LENGTH as MAX_LONG_DESC_LENGTH,
)

# This function will be released in zimscraperlib 3.1.2
# Until then, it is forked here for convenience


def compute_descriptions(
    default_description: str,
    user_description: str | None,
    user_long_description: str | None,
) -> tuple[str, str | None]:
    """Computes short and long descriptions compliant with ZIM standard.

    Based on provided parameters, the function computes a short and a long description
    which are compliant with the ZIM standard (in terms of length).

    User description(s) are used if set. They are checked to not exceed ZIM standard
    maximum length ; an error is thrown otherwise ; if ok, they are returned.

    If user_description is not set, the description is computed based on the default
    description, truncated if needed.

    If user_long_description is not set and default description is too long for the
    description field, the long_description is computed based on the default description
    (truncated if needed), otherwise no long description is returned.

    args:
        default_description:   the description which will be used if user descriptions
                               are not set (typically fetched online)
        user_description:      the description set by the user (typically set by a
                               CLI argument)
        user_long_description: the long description set by the user (typically set by a
                               CLI argument)

    Returns a tuple of (description, long_description)
    """

    if user_description and len(user_description) > MAX_DESC_LENGTH:
        raise ValueError(
            f"Description too long ({len(user_description)}>{MAX_DESC_LENGTH})"
        )
    if user_long_description and len(user_long_description) > MAX_LONG_DESC_LENGTH:
        raise ValueError(
            f"LongDescription too long ({len(user_long_description)}"
            f">{MAX_LONG_DESC_LENGTH})"
        )

    if not user_long_description and len(default_description) > MAX_DESC_LENGTH:
        user_long_description = default_description[0:MAX_LONG_DESC_LENGTH]
        if len(default_description) > MAX_LONG_DESC_LENGTH:
            user_long_description = user_long_description[:-1] + "…"
    if not user_description:
        user_description = default_description[0:MAX_DESC_LENGTH]
        if len(default_description) > MAX_DESC_LENGTH:
            user_description = user_description[:-1] + "…"

    return (user_description, user_long_description)
